/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_MEMORIZEIMPL_H_
#define AIDGE_CPU_OPERATOR_MEMORIZEIMPL_H_

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Memorize.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"
#include "aidge/backend/cpu/data/GetCPUPtr.h"
#include <memory>
#include <vector>

namespace Aidge {
class MemorizeImpl_cpu : public OperatorImpl {
public:
    MemorizeImpl_cpu(const Memorize_Op& op) : OperatorImpl(op, "cpu") {}

    static std::unique_ptr<MemorizeImpl_cpu> create(const Memorize_Op& op) {
        return std::make_unique<MemorizeImpl_cpu>(op);
    }

    Elts_t getNbRequiredData(const IOIndex_t inputIdx) const override final;
    Elts_t getRequiredMemory(const Aidge::IOIndex_t outputIdx,
                               const std::vector<Aidge::DimSize_t> &/*inputsSize*/) const override final;
    void updateConsummerProducer() override final;
    void forward() override;
};

namespace {
static Registrar<Memorize_Op> registrarMemorizeImpl_cpu("cpu", Aidge::MemorizeImpl_cpu::create);
}
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_MEMORIZEIMPL_H_ */
