/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include "aidge/backend/cpu/operator/GatherImpl.hpp"

#include <memory>
#include <vector>

#include "aidge/backend/cpu/operator/GatherImpl_forward_kernels.hpp"
#include "aidge/data/Data.hpp"
#include "aidge/data/Tensor.hpp"
#include "aidge/operator/Gather.hpp"
#include "aidge/utils/Types.h"

void Aidge::GatherImpl_cpu::forward() {
    const Gather_Op& op = static_cast<const Gather_Op&>(mOp);

    auto kernelFunc = Registrar<GatherImplForward_cpu>::create({
                            op.getInput(0)->dataType(),
                            op.getOutput(0)->dataType()
                        });

    // Call kernel
    kernelFunc(dynamic_cast<const Gather_Op&>(mOp).getStaticAttributes(),
            op.getInput(0)->dims(),
            op.getInput(0)->getImpl()->rawPtr(),
            op.getOutput(0)->getImpl()->rawPtr()
        );
}
