/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cassert>
#include <chrono>  // std::chrono::milliseconds
#include <numeric> // std::accumulate
#include <thread>  // std::this_thread::sleep_for
#include <vector>

#include "aidge/operator/Pop.hpp"
#include "aidge/utils/Types.h"
#include "aidge/backend/cpu/data/GetCPUPtr.h"

#include "aidge/backend/cpu/operator/PopImpl.hpp"

Aidge::Elts_t Aidge::PopImpl_cpu::getNbRequiredData(const Aidge::IOIndex_t inputIdx) const {
    assert(mOp.getRawInput(inputIdx) && "requires valid input");

    return Elts_t::DataElts(std::static_pointer_cast<Tensor>(mOp.getRawInput(inputIdx))->size()
        / std::static_pointer_cast<Tensor>(mOp.getRawInput(inputIdx))->dims()[0]);
}

void Aidge::PopImpl_cpu::forward() {
    assert(std::static_pointer_cast<Tensor>(mOp.getRawInput(0)) && "missing input #0");

    const Pop_Op& op = dynamic_cast<const Pop_Op&>(mOp);
    const unsigned int forwardStep = op.template getAttr<PopAttr::ForwardStep>();

    *std::static_pointer_cast<Tensor>(mOp.getRawOutput(0))
        = std::static_pointer_cast<Tensor>(mOp.getRawInput(0))->extract({forwardStep});
}
